using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Reflection;
using System.IO;
using System.ComponentModel;

namespace SodaMachine
{
    public class LcdDisplay : System.Windows.Forms.Control
    {
        protected decimal _value = 0.0m;
        protected Bitmap[] _bitmaps = null;

        protected const decimal MinValue = 0.0m;
        protected const decimal MaxValue = 1.25m;

        public LcdDisplay()
        {
            InitializeComponent();
        }

        protected void InitializeComponent()
        {
            // Set up double buffering for control redraws
            this.SetStyle(ControlStyles.DoubleBuffer, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.UserPaint, true);

            // Size of the bitmap when containing "$0.00" plus 2 pixel border each side
            this.Size = new Size(81, 27);

            // Load bitmaps. They're not the same width, so we can't use a single
            // bitmap and offset to the digit we want. We'd have to keep a separate
            // array of bitmap widths. This way each bitmap "knows" how wide it is,
            // but we do have to deal with separate bitmaps. 
            _bitmaps = new Bitmap[12];
            Assembly assem = Assembly.GetExecutingAssembly();
            Stream stream = assem.GetManifestResourceStream("SodaMachine.Resources.Zero.bmp");
            _bitmaps[0] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.One.bmp");
            _bitmaps[1] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Two.bmp");
            _bitmaps[2] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Three.bmp");
            _bitmaps[3] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Four.bmp");
            _bitmaps[4] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Five.bmp");
            _bitmaps[5] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Six.bmp");
            _bitmaps[6] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Seven.bmp");
            _bitmaps[7] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Eight.bmp");
            _bitmaps[8] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Nine.bmp");
            _bitmaps[9] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Decimal.bmp");
            _bitmaps[10] = new Bitmap(stream);
            stream = assem.GetManifestResourceStream("SodaMachine.Resources.Dollar.bmp");
            _bitmaps[11] = new Bitmap(stream);
        }

        protected override void  Dispose(bool disposing)
        {
            // Release bitmaps
            if (disposing)
            {
                foreach (Bitmap bmp in _bitmaps)
                {
                    // Dispose of this bitmap
                    bmp.Dispose();
                } // foreach
            } // if

 	        base.Dispose(disposing);
        }

        [DefaultValueAttribute("")]
        [DescriptionAttribute("Monetary value associated this control.")]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content | DesignerSerializationVisibility.Visible)]
        public decimal Value
        {
            get { return _value; }
            set
            {
                // Persist value
                if (value > MaxValue) _value = MaxValue;
                else if (value < MinValue) _value = MinValue;
                else _value = value;

                // Redraw
                Refresh();
            }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            // Clear background
            Graphics gfx = e.Graphics;
            gfx.FillRectangle(Brushes.Black, ClientRectangle);

            // Draw border. Dark left and top, light right and bottom
            Pen pen = null;
            using (pen = new Pen(SystemColors.ControlLightLight))
            {
                gfx.DrawLine(pen, ClientRectangle.Width - 1, 0, ClientRectangle.Width - 1, ClientRectangle.Height - 1); // right
                gfx.DrawLine(pen, ClientRectangle.Width - 1, ClientRectangle.Height - 1, 0, ClientRectangle.Height - 1); // bottom
            } // using
            using (pen = new Pen(SystemColors.ControlDark))
            {
                gfx.DrawLine(pen, ClientRectangle.Width, 0, 0, 0); // top
                gfx.DrawLine(pen, 0, 0, 0, ClientRectangle.Height); // left
            } // using

            // Draw digits. We convert the value into a string since it's
            // easier to index the individual digit as a character. We can
            // also have the string formatted for us, which is nice.
            string digits = Value.ToString("C"); // works for '$', not so well for international values
            Int32 nextX = 2; // start at second pixel
            foreach (char digit in digits)
            {
                // Place this digit/indicator
                Int32 offset = digit - '0';
                if (offset == -2) offset = 10; // '.'
                else if (offset == -12) offset = 11; // '$'

                Bitmap bmp = _bitmaps[offset];
                gfx.DrawImage(bmp, nextX, 2);
                nextX += bmp.Width;
            } // foreach
        }
    }
}
